<?php
/**
 * Fichier gérant l'installation et désinstallation du plugin Territoires.
 *
 * @package    SPIP\TERRITOIRES\INSTALLATION
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Fonction d'installation et de mise à jour du plugin Territoires.
 *
 * @param string $nom_meta_base_version Nom de la meta informant de la version du schéma de données du plugin
 *                                      installé dans SPIP
 * @param string $version_cible         Version du schéma de données dans ce plugin (déclaré dans paquet.xml)
 *
 * @return void
**/
function territoires_upgrade(string $nom_meta_base_version, string $version_cible) : void {
	$maj = [];

	// Initialisation de la configuration
	// -- Configuration statique du plugin Territoires
	include_spip('inc/territoire');
	$config_statique = territoire_configurer('statique');
	// -- Configuration dynamique du plugin Territoires
	$config_modifiable = territoire_configurer('utilisateur');

	$maj['create'] = [
		[
			'maj_tables',
			['spip_territoires', 'spip_territoires_extras', 'spip_territoires_liens']
		],
		[
			'ecrire_config',
			'territoires',
			array_merge($config_statique, $config_modifiable)
		]
	];

	// Ajout des extras pour les zones (code geoIP des continents).
	$maj['2'] = [
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Ajout de la colonne profondeur_type et de la configuration associée pour son importation de Nomenclatures.
	$maj['3'] = [
		[
			'sql_alter',
			'TABLE spip_territoires ADD COLUMN profondeur_type int DEFAULT 0 NOT NULL AFTER iso_parent'
		],
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Ajout de la liste des types de territoire
	$maj['4'] = [
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Ajout des coordonnées géographiques comme extras des infrasubdivisions
	$maj['5'] = [
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Ajout des coordonnées géographiques comme extras des infrasubdivisions
	$maj['6'] = [
		[
			'sql_alter',
			'TABLE spip_territoires ADD COLUMN iso_parent_alt varchar(20) NOT NULL DEFAULT "" AFTER profondeur_type'
		],
		['sql_alter', 'TABLE spip_territoires ADD INDEX iso_parent_alt (iso_parent_alt)'],
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Changement d'un nom de nature d'extra (date_creation)
	$maj['7'] = [
		[
			'adapter_config_statique',
			$config_statique
		]
	];

	// Création des tables et de la configuration du plugin
	include_spip('base/upgrade');
	maj_plugin($nom_meta_base_version, $version_cible, $maj);

	// Peuplement minimal du plugin en synchrone:
	// - les zones du monde
	// - les pays
	// - les subdivisions françaises
	$options['extras'] = ['code', 'info'];
	territoire_peupler('zone', '', $options);
	territoire_peupler('country', '', $options);
	territoire_peupler('subdivision', 'FR', $options);
	// Peuplement du plugin en asynchrone:
	// - les infrasubdivisions françaises
	territoire_peupler_asynchrone('infrasubdivision', 'FR', $options);
}

/**
 * Fonction de désinstallation du plugin Territoires.
 *
 * @param string $nom_meta_base_version Nom de la meta informant de la version du schéma de données du plugin
 *                                      installé dans SPIP
 *
 * @return void
**/
function territoires_vider_tables(string $nom_meta_base_version) : void {
	// Supprimer les tables ajoutées par le plugin
	sql_drop_table('spip_territoires');
	sql_drop_table('spip_territoires_extras');
	sql_drop_table('spip_territoires_liens');

	// Nettoyer les liens courants (le génie optimiser_base_disparus se chargera de nettoyer toutes les tables de liens)
	sql_delete('spip_documents_liens', sql_in('objet', ['territoire']));
	sql_delete('spip_mots_liens', sql_in('objet', ['territoire']));
	sql_delete('spip_auteurs_liens', sql_in('objet', ['territoire']));
	// Nettoyer les versionnages
	sql_delete('spip_versions', sql_in('objet', ['territoire']));
	sql_delete('spip_versions_fragments', sql_in('objet', ['territoire']));
	sql_delete('spip_forum', sql_in('objet', ['territoire']));

	// Effacer les meta du plugin
	effacer_meta('territoires');
	effacer_meta('territoires_peuplement');
	effacer_meta($nom_meta_base_version);
}

/**
 * Fonction de mise à jour de la config statique du plugin.
 *
 * @param array $config_statique Nouvelle configuration statique
 *
 * @return void
**/
function adapter_config_statique(array $config_statique) : void {
	// Configuration actuelle pour conserver la config modifiable
	include_spip('inc/config');
	$config = lire_config('territoires');

	// On insère la nouvelle config statique
	foreach ($config_statique as $_cle => $_config) {
		$config[$_cle] = $_config;
	}

	// Mise à jour en meta
	ecrire_config('territoires', $config);
}
